import dataclasses
import typing

import parso

from odd.ast_utils import get_model_name

from ._base import Artifact
from .addon import AddonPath
from .issue import Position


@dataclasses.dataclass
class PythonModule(AddonPath):
    node: parso.python.tree.Module


@dataclasses.dataclass
class FieldArg:
    value: typing.Any
    start_pos: Position
    end_pos: Position


@dataclasses.dataclass
class FieldKwarg:
    name: str
    value: typing.Any
    start_pos: Position
    end_pos: Position


@dataclasses.dataclass
class ModelDefinition(AddonPath):
    node: parso.tree.Node
    class_name: str
    params: typing.Dict[str, typing.Any] = dataclasses.field(default_factory=dict)
    bases: typing.List = dataclasses.field(default_factory=list)

    @property
    def name(self) -> typing.Optional[str]:
        return get_model_name(self.params)


@dataclasses.dataclass
class FieldDefinition(Artifact):
    name: str
    class_name: str
    model: ModelDefinition
    start_pos: Position
    end_pos: Position
    args: typing.List[FieldArg] = dataclasses.field(default_factory=list)
    kwargs: typing.List[FieldKwarg] = dataclasses.field(default_factory=list)
